@@ -140,8 +140,8 @@ class Node(object):
 
     def _annotate_with_default_opts(self, options):
         options['-n'] = self.name
-        self._setdefaultopt(options, ['--pidfile', '-p'], '%n.pid')
-        self._setdefaultopt(options, ['--logfile', '-f'], '%n%I.log')
+        self._setdefaultopt(options, ['--pidfile', '-p'], '/var/run/celery/%n.pid')
+        self._setdefaultopt(options, ['--logfile', '-f'], '/var/log/celery/%n%I.log')
         self._setdefaultopt(options, ['--executable'], sys.executable)
         return options
 
@@ -151,7 +151,11 @@ class Node(object):
                 return d[opt]
             except KeyError:
                 pass
-        return d.setdefault(alt[0], value)
+        value = d.setdefault(alt[0], os.path.normpath(value))
+        dir_path = os.path.dirname(value)
+        if not os.path.exists(dir_path):
+            os.makedirs(dir_path)
+        return value
 
     def _prepare_expander(self):
         shortname, hostname = self.name.split('@', 1)
