@@ -7,7 +7,7 @@ import os
 
 from kombu.utils.encoding import ensure_bytes
 
-from celery import uuid
+from celery import uuid, states
 from celery.backends.base import KeyValueStoreBackend
 from celery.exceptions import ImproperlyConfigured
 
@@ -58,18 +58,23 @@ class FilesystemBackend(KeyValueStoreBackend):
         # Lets verify that we've everything setup right
         self._do_directory_test(b'.fs-backend-' + uuid().encode(encoding))
 
+    def __reduce__(self, args=(), kwargs={}):
+        kwargs.update(
+            dict(url=self.url))
+        return super(FilesystemBackend, self).__reduce__(args, kwargs)
+
     def _find_path(self, url):
         if not url:
             raise ImproperlyConfigured(E_NO_PATH_SET)
-        if url.startswith('file:///'):
-            return url[7:]
         if url.startswith('file://localhost/'):
             return url[16:]
+        if url.startswith('file://'):
+            return url[7:]
         raise ImproperlyConfigured(E_PATH_NON_CONFORMING_SCHEME)
 
     def _do_directory_test(self, key):
         try:
-            self.set(key, b'test value')
+            self.set(key, b'test value', states.SUCCESS)
             assert self.get(key) == b'test value'
             self.delete(key)
         except IOError:
@@ -85,7 +90,7 @@ class FilesystemBackend(KeyValueStoreBackend):
         except FileNotFoundError:
             pass
 
-    def set(self, key, value):
+    def set(self, key, value, state):
         with self.open(self._filename(key), 'wb') as outfile:
             outfile.write(ensure_bytes(value))
 
