@@ -141,7 +141,7 @@ class RiakBackend(KeyValueStoreBackend):
     def get(self, key):
         return self.bucket.get(key).data
 
-    def set(self, key, value):
+    def set(self, key, value, state):
         _key = self.bucket.new(key, data=value)
         _key.store()
 
