@@ -408,8 +408,12 @@ class Signature(dict):
             other = maybe_unroll_group(other)
             if isinstance(self, _chain):
                 # chain | group() -> chain
+                tasks = self.unchain_tasks()
+                if not tasks:
+                    # If the chain is empty, return the group
+                    return other
                 return _chain(seq_concat_item(
-                    self.unchain_tasks(), other), app=self._app)
+                    tasks, other), app=self._app)
             # task | group() -> chain
             return _chain(self, other, app=self.app)
 
@@ -622,7 +626,7 @@ class _chain(Signature):
         return signature
 
     def unchain_tasks(self):
-        # Clone chain's tasks assigning sugnatures from link_error
+        # Clone chain's tasks assigning signatures from link_error
         # to each task
         tasks = [t.clone() for t in self.tasks]
         for sig in self.options.get('link_error', []):
@@ -878,7 +882,9 @@ class chain(_chain):
         if not kwargs and tasks:
             if len(tasks) != 1 or is_list(tasks[0]):
                 tasks = tasks[0] if len(tasks) == 1 else tasks
-                return reduce(operator.or_, tasks)
+                # if is_list(tasks) and len(tasks) == 1:
+                #     return super(chain, cls).__new__(cls, tasks, **kwargs)
+                return reduce(operator.or_, tasks, chain())
         return super(chain, cls).__new__(cls, *tasks, **kwargs)
 
 
