@@ -16,7 +16,7 @@ from __future__ import absolute_import, unicode_literals
 from .utils.dispatch import Signal
 
 __all__ = (
-    'before_task_publish', 'after_task_publish',
+    'before_task_publish', 'after_task_publish', 'task_internal_error',
     'task_prerun', 'task_postrun', 'task_success',
     'task_retry', 'task_failure', 'task_revoked', 'celeryd_init',
     'celeryd_after_setup', 'worker_init', 'worker_process_init',
@@ -65,6 +65,12 @@ task_failure = Signal(
         'task_id', 'exception', 'args', 'kwargs', 'traceback', 'einfo',
     },
 )
+task_internal_error = Signal(
+    name='task_internal_error',
+    providing_args={
+        'task_id', 'args', 'kwargs', 'request', 'exception', 'traceback', 'einfo'
+    }
+)
 task_revoked = Signal(
     name='task_revoked',
     providing_args={
