@@ -1,10 +1,12 @@
 """Consumer Broker Connection Bootstep."""
 from __future__ import absolute_import, unicode_literals
+
 from kombu.common import ignore_errors
+
 from celery import bootsteps
 from celery.utils.log import get_logger
 
-__all__ = ['Connection']
+__all__ = ('Connection',)
 
 logger = get_logger(__name__)
 info = logger.info
